# Supports Organizer.rb
=begin rdoc
= Similar Groups
Copyright 2009, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement. It is not ok to distribute this code without the Author's consent.
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.sketchucation.com  - This script will probably be removed and made private.
Name:: Similar Groups
Version:: v0.1
SU Version:: Developed on SU 7, should be compatible with 6 and 5 at least.
Date:: Novermber 1, 2009
Description:: This script finds all groups and compares them to see if they are similar enough to be made into a component
Usage:: Run the script from Plugins > Chris Fullmer Tools > Similar Groups ( TEST ).  If the user has geometry selected, it will work the selection.  If nothing is slected, it will work with all loose geometry.  It will not group components or existing groups.  It makes a new group for all connected geometry.  So if there are 20 separate boxes, it will make 20 separate groups - 1 for each box.  I use this to quickly group large amounts of quickly made 3d buildings over bery large areas of land.  This way I do not have so much loose geometry floating around.
History::
0.1:: 2009-02-07
   * First release
   * Just testing some ideas.  
=end


require 'sketchup.rb'

module Clf_Similar_Groups
def Clf_Similar_Groups.simcomp
	model = Sketchup.active_model
	sel = model.selection
	ents = model.active_entities
	groups = []
	
	a = Sketchup.version
	b = a[0,1].to_i
	if b > 6
		model.start_operation("Similar Group Finder", true)
	else
		model.start_operation("Similar Group Finder")
	end
	
	# If the user has a selection, we'll use it.  If not, we iterate through the entire model
	if sel.length > 0
		to_test = sel
	else
		to_test = ents
	end

	# Find all the groups in the selection/model
	to_test.each { |e| groups << e if e.is_a? Sketchup::Group }

	pos_group = []
	temp_pos_group = []
	push = false

	#This section loops through the groups array.  It removes any groups found to be similar, then re-loops.
	c=0
	while groups.length > 0
		base = groups[0]
		sel.add( base )
		temp_pos_group << base
		groups.each do |test|
			sel.add test
			similar = Clf_Similar_Groups.first_compare( base, test )
			if test != base
				temp_pos_group << test if similar
			end
			sel.remove test
		end
		temp_pos_group - [base]
		push = true if temp_pos_group.length > 1
		pos_group[c] = temp_pos_group if push
		groups = groups - [base]
		groups = groups	- pos_group[c] if push
		temp_pos_group = []
		c+=1 if push
		push = false
	end
	
	# This section goes through the array of similar groups and adds a text label to indicate which 
	# were found to be similar to each other.
	c=0
	pos_group.each do |e|
		e.each do |ee|
			cen = ee.bounds.center
			height =  ee.bounds.depth / 2
			pt = cen
			pt[2] = pt[2] + height
			ents.add_text c.to_s, pt
		end
	c = c+1
	end
	model.commit_operation
end # method

# This Compares two groups for certain similarities.
# Returns true if similar, false otherwise.
def Clf_Similar_Groups.first_compare( base, test )
	#Set empty variables and arrays
	b_ents_length = base.entities.length
	b_edges = []
	b_e_lengths = []
	t_edges = []
	t_e_lengths = []
	b_faces = []
	t_faces = []
	b_faces_area = 0
	t_faces_area = 0
	similar = true
	
	# Find all values for the base groups
	base.entities.each do |e|
		if e.is_a? Sketchup::Edge
			b_edges << e
			b_e_lengths << e.length
		end
		b_faces << e if e.is_a? Sketchup::Face
	end
	b_e_lengths.sort!
	b_faces.each do |e|
		b_faces_area = t_faces_area + e.area
	end
	
	# Find all values for the test group
	t_ents_length = test.entities.length
	test.entities.each do |e|
		if e.is_a? Sketchup::Edge
			t_edges << e
			t_e_lengths << e.length
		end
		t_faces << e if e.is_a? Sketchup::Face
	end
	t_e_lengths.sort!
	t_faces.each do |e|
		t_faces_area = t_faces_area + e.area
	end

	# Compare values, if something does not match, similar is set as false.
	similar = false if b_ents_length != t_ents_length
	similar = false if b_edges.length != t_edges.length
	#similar = false if b_ents_length != t_ents_length
	similar = false if b_e_lengths != t_e_lengths
	similar = false if b_faces.length != t_faces.length
	#similar = false if ((b_faces_area - t_faces_area).abs  < 0.00001) #unstable for now, face areas do not compare well at all!
	similar	# the value that is returned
	
end # method
end #module




# ------ { MENU SYSTEM } ------
if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__) then
		organizer.add_item("Similar Groups (TEST)") { Clf_Similar_Groups.simcomp }
	end
else
	if !file_loaded?('clf_menu_loader') then
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
	if !file_loaded?(__FILE__) then
		$clf_tools_menu.add_item("Similar Groups (TEST)") { Clf_Similar_Groups.simcomp  }
	end
end

file_loaded('clf_menu_loader')
file_loaded(__FILE__)